#ifdef DEBUG

// The MEMDATA structure is used to create
// a linked list through which mtlib tracks
// memory blocks as they are allocated.

typedef struct __TAG_MEMDATA
{
	unsigned long	id;
	size_t		size;
	void		*pointer;

	char		*filename;
	long		line;

	void		*next;
} MEMDATA;

// The MT_CONTEXT structure is used to
// maintain a context for a particular
// thread. You should create a new context
// for each thread and only use it in that
// particular thread.

typedef struct __TAG_MT_CONTEXT
{
	MEMDATA *head;
	unsigned long counter;
	char *name;
} MT_CONTEXT;

// These functions act as wrappers for the standard
// library. You should not use them directly in your
// code. Use the macros below instead.

void *mt_malloc (MT_CONTEXT *context, size_t size, char *filename, long line);
void *mt_calloc (MT_CONTEXT *context, size_t items, size_t size, char *filename, long line);
void *mt_realloc (MT_CONTEXT *context, void *ptr, size_t size, char *filename, long line);
void *mt_free (MT_CONTEXT *context, void *ptr);

MT_CONTEXT *mt_initialize(char *context_name);
void mt_terminate(MT_CONTEXT *context);

// The following should be the first line at the beginning of your
// each function in which you create a new context.

#define		MT_DECLARE_DATA		MT_CONTEXT	*__mt_context;

// MT_CALL_DATA should be at the beginning of each call to a function
// that uses the memory allocation functions but does not create a new 
// context. Use MT_CALL_DATA_Cf the function requires additional parameters.

#define		MT_CALL_DATA		__mt_context
#define		MT_CALL_DATA_C		MT_CALL_DATA,

// MT_CONTEXT_DATA should be the first item in the declaration of any function
// that uses the memory allocation functions but does not create a new context.
// Use MT_CONTEXT_DATA_C if the function has more one or more other parameters.

#define		MT_CONTEXT_DATA		MT_CONTEXT	*__mt_context
#define		MT_CONTEXT_DATA_C	MT_CONTEXT_DATA,

// This macro creates a new context. The context can be given a particular name 
// to facilitate the process of locating leaks

#define		MT_INITIALIZE(name)		__mt_context = mt_initialize(name);

// This macro destroys a context and outputs any memory leaks generated in it.

#define		MT_TERMINATE			mt_terminate(__mt_context);

// The following are wrappers to the normal stdlib functions. They are declared
// and behave exactly like their usual counterparts.

#define		MT_MALLOC(size)			mt_malloc(__mt_context,size,__FILE__,__LINE__)
#define		MT_CALLOC(size,items)		mt_calloc(__mt_context,size,items,__FILE__,__LINE__)
#define		MT_REALLOC(ptr,size)		mt_realloc(__mt_context,ptr,size,__FILE__,__LINE__)
#define		MT_FREE(ptr)			mt_free(__mt_context,ptr)

#else

// In non-debug mode, the macros are redefined so that mtfree is completely out of the picture

#define		MT_DECLARE_DATA		
#define		MT_CALL_DATA		
#define		MT_CALL_DATA_C		
#define		MT_CONTEXT_DATA		
#define		MT_CONTEXT_DATA_C	

#define		MT_INITIALIZE(name)		
#define		MT_TERMINATE			
#define		MT_MALLOC(size)			malloc(size)
#define		MT_CALLOC(size,items)		calloc(size,items)
#define		MT_REALLOC(ptr,size)		realloc(ptr,size)
#define		MT_FREE(ptr)			free(ptr)

#endif


