################################################################################
#
# Generates a single C-file with many includes, where each include is included
# by its predecessor in the C-file. Hence, this represents a test of include
# guard efficiency
#
# Updated: 17th June 2003

import sys
import string

NUM_FUNCTIONS = 100
out_fname     = 'comp3\\comp3.c'
include_base  = 'comp3\\inc'
source_base   = 'comp3\\inc'

########################################
# Process the main file

if len(sys.argv) > 1:
  NUM_FUNCTIONS = int(sys.argv[1])

print >> sys.stderr, "Generating comp3 scenario with %d functions" % NUM_FUNCTIONS

print 'Creating output file %s' % (out_fname)
sys.stdout = open(out_fname, 'w')

print "/* /////////////////////////////////////////////////////////////////////////////"
print " * File :   %s" % (out_fname)
print " *"
print " * Created: 0th January 2100"
print " * Updated: 0th January 2100"
print " *"
print " * ////////////////////////////////////////////////////////////////////////// */"
print ""
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Includes"
print " */"
print ""
for i in xrange(0, NUM_FUNCTIONS):
	print '#include "inc%04d.h"' % ((NUM_FUNCTIONS - 1) - i)
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Functions"
print " */"
print ""
print "int process(int i, int j)"
print "{"
print "  int result = 0;"
print ""
for i in xrange(0, NUM_FUNCTIONS):
	print "  result ^= fn%04d(i, j + result);" % (i)
print ""
print "  return result;"
print "}"
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Functions"
print " */"
print ""
print "int main()"
print "{"
print "  char x;"
print "  char y;"
print "  int  i = process((int)&x, (int)&x);"
print ""
print "  return i;"
print "}"
print ""
print "/* ////////////////////////////////////////////////////////////////////////// */"

########################################
# Now do the include files

for i in xrange(0, NUM_FUNCTIONS):
  include_name  = "%s%04d.h" % (include_base, i)
  inc_guard     = "INCL_" + string.upper(include_name)
  inc_guard     = string.replace(inc_guard, "\\", "_")
  inc_guard     = string.replace(inc_guard, ".", "_")
  sys.stdout = open(include_name, 'w')
  print "/* /////////////////////////////////////////////////////////////////////////////"
  print " * File :   %s" % (include_name)
  print " *"
  print " * Created: 0th January 2100"
  print " * Updated: 0th January 2100"
  print " *"
  print " * ////////////////////////////////////////////////////////////////////////// */"
  print ""
  print "#ifndef " + inc_guard
  print "#define " + inc_guard
  print ""
  print "/* /////////////////////////////////////////////////////////////////////////////"
  print " * Includes"
  print " */"
  print ""
  print '#include "inc%04d.h"' % (i)
  if i > 0:
    print '#include "inc%04d.h"' % (i - 1)
  print ""
  print "int fn%04d(int i, int j)" % (i)
  print "{"
  index = i % 5;
  if index == 0:
    print "  return i & j;"
  elif index == 1:
    print "  return i ^ j;"
  elif index == 2:
    print "  return i << (j & 3);"
  elif index == 3:
    print "  return i ^ (int)(&j);"
  elif index == 4:
    print "  return i + (j - 200);"
  print "}"
  print ""
  print "/* ////////////////////////////////////////////////////////////////////////// */"
  print ""
  print "#endif /* !%s */" % (inc_guard)
  print ""
  print "/* ////////////////////////////////////////////////////////////////////////// */"
