################################################################################
#
# Generates a PCH header & source files, and a set of reliant source files.
# 
# 
#
# Updated: 21st June 2003

import sys
import string

NUM_FUNCTIONS = 40#00
pch_h_name		=	'comp4\\pch.h'
pch_cpp_name	=	'comp4\\pch.cpp'
main_name		=	'comp4\\main.cpp'
include_base	=	'comp4\\cls'
source_base		=	'comp4\\cls'
makefile_bc56	=	'comp4\\makefile.bc56'
makefile_cw8	=	'comp4\\makefile.cw8'
makefile_dm834	=	'comp4\\makefile.dm834'
makefile_gcc32	=	'comp4\\makefile.gcc32'
makefile_icl7	=	'comp4\\makefile.icl7'
makefile_vc5	=	'comp4\\makefile.vc5'
makefile_vc6	=	'comp4\\makefile.vc6'
makefile_vc7	=	'comp4\\makefile.vc7'
makefile_vc71	=	'comp4\\makefile.vc71'
makefile_wc12 = 'comp4\\makefile.wc12'

########################################
#

def make_guard(s):
	s	=	"INCL_" + string.upper(s)
	s	=	string.replace(s, "\\", "_")
	s	=	string.replace(s, ".", "_")
	return s

########################################
# Process the main file

if len(sys.argv) > 1:
  NUM_FUNCTIONS = int(sys.argv[1])

print >> sys.stderr, "Generating comp4 scenario with %d functions" % NUM_FUNCTIONS

sys_stdout	=	sys.stdout;

## PCH.H

sys.stdout = sys_stdout
print 'Creating pre-compiled header file %s' % (pch_h_name)
sys.stdout = open(pch_h_name, 'w')

print "/* /////////////////////////////////////////////////////////////////////////////"
print " * File :   %s" % (pch_h_name)
print " *"
print " * Created: 0th January 2100"
print " * Updated: 0th January 2100"
print " *"
print " * ////////////////////////////////////////////////////////////////////////// */"
print ""
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Includes"
print " */"
print ""
print "#include <windows.h>"
print "#include <stdio.h>"
print "#include <stdlib.h>"
print "#include <malloc.h>"
print ""
print "/* ////////////////////////////////////////////////////////////////////////// */"


## PCH.CPP

sys.stdout = sys_stdout
print 'Creating pre-compiled header file %s' % (pch_cpp_name)
sys.stdout = open(pch_cpp_name, 'w')

print "/* /////////////////////////////////////////////////////////////////////////////"
print " * File :   %s" % (pch_cpp_name)
print " *"
print " * Created: 0th January 2100"
print " * Updated: 0th January 2100"
print " *"
print " * ////////////////////////////////////////////////////////////////////////// */"
print ""
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Includes"
print " */"
print ""
print '#include "pch.h"'
print ""
print "/* ////////////////////////////////////////////////////////////////////////// */"


## MAIN.CPP

sys.stdout = sys_stdout
print 'Creating output file %s' % (main_name)
sys.stdout = open(main_name, 'w')

print "/* /////////////////////////////////////////////////////////////////////////////"
print " * File :   %s" % (main_name)
print " *"
print " * Created: 0th January 2100"
print " * Updated: 0th January 2100"
print " *"
print " * ////////////////////////////////////////////////////////////////////////// */"
print ""
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Includes"
print " */"
print ""
print '#include "pch.h"'
for i in xrange(0, NUM_FUNCTIONS):
	print '#include "cls%04d.h"' % ((NUM_FUNCTIONS - 1) - i)
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Functions"
print " */"
print ""
print "int process(IStream *node, IStream *pstm, UINT u)"
print "{"
print "  int result = 0;"
print ""
for i in xrange(0, NUM_FUNCTIONS):
	print "  result += SUCCEEDED(Cls%04d(node).Method(pstm, u));" % (i)
print ""
print "  return result;"
print "}"
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Statics"
print " */"
print ""
print "static IID _IID_IXMLDOMDocument	=	{	0xF52E2B61, 0x18A1, 0x11d1, { 0xB1, 0x05, 0x00, 0x80, 0x5F, 0x49, 0x91, 0x6B } };"
print "#define	IID_IXMLDOMDocument	_IID_IXMLDOMDocument"
print "static CLSID _CLSID_XMLDocument =	{	0xCFC399AF, 0xD876, 0x11d0, { 0x9C, 0x10, 0x00, 0xC0, 0x4F, 0xC9, 0x9C, 0x8E } };"
print "#define	CLSID_XMLDocument	_CLSID_XMLDocument"
print ""
print "/* /////////////////////////////////////////////////////////////////////////////"
print " * Functions"
print " */"
print ""
print "int main()"
print "{"
print "  int     iRet  = 1;"
print "  IStream *pstm;"
print "  HRESULT hr    = ::CreateStreamOnHGlobal(NULL, false, &pstm);"
print ""
print "  if(SUCCEEDED(hr))"
print "  {"
print "    IStream *pClone;"
print ""
print "    hr = pstm->Clone(&pClone);"
print ""
print "    if(SUCCEEDED(hr))"
print "    {"
print "      iRet = process(pClone, pstm, 10);"
print ""
print "      pClone->Release();"
print "    }"
print ""
print "    pstm->Release();"
print "  }"
print ""
print "  return iRet;"
print "}"
print ""
print "/* ////////////////////////////////////////////////////////////////////////// */"

########################################
# Now do the include files

for i in xrange(0, NUM_FUNCTIONS):
  sys.stdout = sys_stdout
  include_name  =	"%s%04d.h" % (include_base, i)
  inc_guard		=	make_guard(include_name)
  sys.stdout	=	open(include_name, 'w')
  print "/* /////////////////////////////////////////////////////////////////////////////"
  print " * File :   %s" % (include_name)
  print " *"
  print " * Created: 0th January 2100"
  print " * Updated: 0th January 2100"
  print " *"
  print " * ////////////////////////////////////////////////////////////////////////// */"
  print ""
  print "#ifndef " + inc_guard
  print "#define " + inc_guard
  print ""
  print "/* /////////////////////////////////////////////////////////////////////////////"
  print " * Includes"
  print " */"
  print ""
  print '#include <windows.h>'
  print ""
  print "/* /////////////////////////////////////////////////////////////////////////////"
  print " * Classes"
  print " */"
  print ""
  print "class Cls%04d" % (i)
  print "{"
  print "public:"
  print "    Cls%04d(IStream *pClone)" % (i)
  print "        : m_node(pClone)"
  print "    {}"
  print ""
  print "public:"
  print "    STDMETHOD(Method)(IStream *pstm, UINT u);"
  print ""
  print "private:"
  print "    IStream *m_node;"
  print "};"
  print ""
  print "/* ////////////////////////////////////////////////////////////////////////// */"
  print ""
  print "#endif /* !%s */" % (inc_guard)
  print ""
  print "/* ////////////////////////////////////////////////////////////////////////// */"

########################################
# Now do the source files

for i in xrange(0, NUM_FUNCTIONS):
  sys.stdout	=	sys_stdout
  source_name	=	"%s%04d.cpp" % (source_base, i)
  sys.stdout	=	open(source_name, 'w')
  print "/* /////////////////////////////////////////////////////////////////////////////"
  print " * File :   %s" % (source_name)
  print " *"
  print " * Created: 0th January 2100"
  print " * Updated: 0th January 2100"
  print " *"
  print " * ////////////////////////////////////////////////////////////////////////// */"
  print ""
  print ""
  print "/* /////////////////////////////////////////////////////////////////////////////"
  print " * Includes"
  print " */"
  print ""
  print '#include "pch.h"'
  print '#include "cls%04d.h"' % (i)
  print ""
  print "/* /////////////////////////////////////////////////////////////////////////////"
  print " * Implementation"
  print " */"
  print ""
  print "STDMETHODIMP Cls%04d::Method(IStream *pstm, UINT /*u*/)" % (i)
  print "{"
  print "  IStream *pClone;"
  print "  HRESULT hr =  pstm->Clone(&pClone);"
  print ""
  print "  if(SUCCEEDED(hr))"
  print "  {"
  print "    pClone->Release();"
  print "  }"
  print ""
  print "  return hr;"
  print "};"
  print ""
  print "/* ////////////////////////////////////////////////////////////////////////// */"


## MAKEFILE.BC56

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_bc56)
sys.stdout = open(makefile_bc56, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_bc56)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	bcc32"
print "LD				=	bcc32"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	kernel32.lib advapi32.lib user32.lib gdi32.lib ole32.lib oleaut32.lib uuid.lib"
print ""
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	-w!"
print "F_OPT_SIZE		=	-O1"
print "F_TARG_PENTIUM	=	-5"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print "PCH_GEN			=	-H -H=$(PCH_FNAME)"
print "PCH_USE			=	-Hu"
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX) -w-inl -w-aus"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print "CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -I$(INC) -I$(INCLUDE)"
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) -o$@ $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -o$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) -e$@ $? $(LIBS)"
print ""
print "################################################################################"
print ""


## MAKEFILE.CW8

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_cw8)
sys.stdout = open(makefile_cw8, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_cw8)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	mwcc"
print "LD				=	mwld"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS				=	-lole32.lib"
print ""
#print "PCH_FNAME		=	.\comp4.pchmw"
print ""
print "F_WARN_AS_ERR	=	-w iserror"
print "F_OPT_SIZE		=	-opt space"
print "F_TARG_PENTIUM	=	-proc 586"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print "PCH_GEN			=	"
print "PCH_USE			=	"
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX)"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print "CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -I- -I."
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
#print "$(PCH_FNAME):"
#print "	$(CPPC) .\pch.h -precompile $@"
print ""
print ".\pch.obj: .\pch.cpp $(PCH_FNAME)"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) .\pch.cpp"
print ""
print ".\main.exe:	$(OBJS_CPP) $(PCH_FNAME)"
print "	$(LD) -o $@ $(OBJS_CPP) $(LIBS)"
print ""
print "################################################################################"
print ""


## MAKEFILE.DM834

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_dm834)
sys.stdout = open(makefile_dm834, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_dm834)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	dmc"
print "LD				=	dmc"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	kernel32.lib advapi32.lib user32.lib gdi32.lib ole32.lib oleaut32.lib uuid.lib"
print ""
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	-wx"
print "F_OPT_SIZE		=	-o+space"
print "F_TARG_PENTIUM	=	-5"
print "F_NO_ERR_MAX	=	-x"
print ""
print ""
print "PCH_GEN			=	-HF$(PCH_FNAME)"
print "PCH_USE			=	-Hu"
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX) #-wc"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print 'CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -I$(INC) -I"$(INCLUDE)"'
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) -o$@ $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -o$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) -o$@ $? $(LIBS)"
print ""
print "################################################################################"
print ""


## MAKEFILE.GCC32

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_gcc32)
sys.stdout = open(makefile_gcc32, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_gcc32)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	g++"
print "LD				=	g++"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	libkernel32.a libadvapi32.a libuser32.a libgdi32.a libole32.a liboleaut32.a libuuid.a"
print ""
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	#"
print "F_OPT_SIZE		=	-O2"
print "F_TARG_PENTIUM	=	#"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print "PCH_GEN			=	"
print "PCH_USE			=	"
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX) #-wc"
print "CPPCDEFS		=	-DNDEBUG -DWIN32 -D_M_IX86"
print ""
print "CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -I$(INC) -I$(INCLUDE)"
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) -o$@ $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -o$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) $(LIBS) -e$@ $?"
print ""
print "################################################################################"
print ""


## MAKEFILE.ICL7

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_icl7)
sys.stdout = open(makefile_icl7, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_icl7)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	icl"
print "LD				=	icl"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	kernel32.lib advapi32.lib user32.lib gdi32.lib ole32.lib oleaut32.lib uuid.lib"
print ""
print "PCH_HDR_FNAME	=	pch.h"
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	-WX"
print "F_OPT_SIZE		=	-O2"
print "F_TARG_PENTIUM	=	-G5"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print 'PCH_GEN			=	-Yc"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print 'PCH_USE			=	-Yu"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX)"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print 'CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -nologo -I$(INC) -I"$(INCLUDE)"'
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) -o$@ $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -o$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) -Fe$@ $(OBJS_CPP) $(LIBS)"
print ""
print "################################################################################"
print ""


## MAKEFILE.VC5

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_vc5)
sys.stdout = open(makefile_vc5, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_vc5)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	cl"
print "LD				=	cl"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	kernel32.lib advapi32.lib user32.lib gdi32.lib ole32.lib oleaut32.lib uuid.lib"
print ""
print "PCH_HDR_FNAME	=	pch.h"
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	-WX"
print "F_OPT_SIZE		=	-O2"
print "F_TARG_PENTIUM	=	-G5"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print 'PCH_GEN			=	-Yc"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print 'PCH_USE			=	-Yu"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX)"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print 'CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -nologo -I$(INC) -I"$(INCLUDE)"'
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) -o$@ $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -o$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) -Fe$@ $(OBJS_CPP) $(LIBS)"
print ""
print "################################################################################"
print ""


## MAKEFILE.VC6

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_vc6)
sys.stdout = open(makefile_vc6, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_vc6)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	cl"
print "LD				=	cl"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	kernel32.lib advapi32.lib user32.lib gdi32.lib ole32.lib oleaut32.lib uuid.lib"
print ""
print "PCH_HDR_FNAME	=	pch.h"
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	-WX"
print "F_OPT_SIZE		=	-O2"
print "F_TARG_PENTIUM	=	-G5"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print 'PCH_GEN			=	-Yc"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print 'PCH_USE			=	-Yu"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX)"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print 'CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -nologo -I$(INC) -I"$(INCLUDE)"'
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) -o$@ $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -o$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) -Fe$@ $(OBJS_CPP) $(LIBS)"
print ""
print "################################################################################"
print ""


## MAKEFILE.VC7

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_vc7)
sys.stdout = open(makefile_vc7, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_vc7)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	cl"
print "LD				=	cl"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	kernel32.lib advapi32.lib user32.lib gdi32.lib ole32.lib oleaut32.lib uuid.lib"
print ""
print "PCH_HDR_FNAME	=	pch.h"
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	-WX"
print "F_OPT_SIZE		=	-O2"
print "F_TARG_PENTIUM	=	-G5"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print 'PCH_GEN			=	-Yc"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print 'PCH_USE			=	-Yu"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX)"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print 'CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -nologo -I$(INC) -I"$(INCLUDE)"'
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) -o$@ $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -o$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) -Fe$@ $(OBJS_CPP) $(LIBS)"
print ""
print "################################################################################"
print ""


## MAKEFILE.VC71

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_vc71)
sys.stdout = open(makefile_vc71, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_vc71)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	cl"
print "LD				=	cl"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	kernel32.lib advapi32.lib user32.lib gdi32.lib ole32.lib oleaut32.lib uuid.lib"
print ""
print "PCH_HDR_FNAME	=	pch.h"
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	-WX"
print "F_OPT_SIZE		=	-O2"
print "F_TARG_PENTIUM	=	-G5"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print 'PCH_GEN			=	-Yc"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print 'PCH_USE			=	-Yu"$(PCH_HDR_FNAME)" -Fp"$(PCH_FNAME)"'
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX)"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print 'CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -c -nologo -I$(INC) -I"$(INCLUDE)"'
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) -o$@ $?"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean | rebuild]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "rebuild: clean target"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -o$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) -Fe$@ $(OBJS_CPP) $(LIBS)"
print ""
print "################################################################################"
print ""


## MAKEFILE.WC12

sys.stdout = sys_stdout
print 'Creating Digital Mars makefile %s' % (makefile_wc12)
sys.stdout = open(makefile_wc12, 'w')

print "# ##############################################################################"
print "# File:         %s" % (makefile_wc12)
print "#"
print "# Purpose:      "
print "#"
print "# Created       "
print "# Updated:      16th June 2003"
print "#"
print "# Copyright:    Synesis Software Pty Ltd, (c) 2003. All rights reserved."
print "#"
print "# Home:         www.synesis.com.au/software"
print "#"
print "# ##############################################################################"
print ""
print "# ##############################################################################"
print "# Macros"
print ""
print "CPPC			=	wpp386"
print "LD				=	wcl386"
print "INC				=	..;. # source directory"
print "LIB				=	-L$(LIB)"
print ""
print "LIBS			=	kernel32.lib advapi32.lib user32.lib gdi32.lib ole32.lib oleaut32.lib uuid.lib"
print ""
print "PCH_HDR_FNAME	=	pch.h"
print "PCH_FNAME		=	comp4.pch"
print ""
print "F_WARN_AS_ERR	=	-we"
print "F_OPT_SIZE		=	-ot"
print "F_TARG_PENTIUM	=	-G5"
print "F_NO_ERR_MAX	=	"
print ""
print ""
print 'PCH_GEN			=	-fh'
print 'PCH_USE			=	-fh'
print "CPPCFLAGS		=	$(F_OPT_SIZE) $(F_TARG_PENTIUM) $(F_NO_ERR_MAX)"
print "CPPCDEFS		=	-DNDEBUG -DWIN32"
print ""
print 'CPPCARGS		=	$(CPPCFLAGS) $(CPPCDEFS) $(F_WARN_AS_ERR) -I$(INC) -I"$(INCLUDE)"'
print ""
print "################################################################################"
print "# Objects"
print ""
print "OBJS_CPP	=	\\"
print "				\\"
print "				.\pch.obj		\\"
print "				.\main.obj		\\"
print "				\\"
for i in xrange(0, NUM_FUNCTIONS):
	print "				.\cls%04d.obj	\\" % (i)
print "				\\"
print ""
print "################################################################################"
print "# Suffix rules"
print ""
print ".cpp.obj:"
print "	$(CPPC) $(PCH_USE) $(CPPCARGS) $? -fo=$@"
print ""
print "################################################################################"
print "# Targets"
print ""
print "usage:"
print '	-@echo "make [target | clean]"'
print ""
print "target:	.\main.exe"
print ""
print "clean:"
print "	@echo Cleaning other file types"
print "	@if exist .\main.exe del .\main.exe"
print "	@del $(OBJS_CPP) 2>NUL"
print "	@if exist *.exe del *.exe"
print "	@if exist *.map del *.map"
print "	@if exist *.tds del *.tds"
print "	@if exist *.obj del *.obj"
print "	@if exist *.err del *.err"
print "	@if exist *.pch del *.pch"
print "	@if exist *.pchi del *.pchi"
print "	@if exist *.dll del *.dll"
print ""
print "# objects - hard-coded, since object files are in separate directory to .cpp"
print ""
print "# executables"
print ""
print ".\pch.obj: .\pch.cpp"
print "	$(CPPC) $(PCH_GEN) $(CPPCARGS) -fo=$@ $?"
print ""
print ".\main.exe:	$(OBJS_CPP)"
print "	$(LD) $? $(LIBS) -fe=$@"
print ""
print "################################################################################"
print ""
