/* /////////////////////////////////////////////////////////////////////////////
 * File:		rectarr.cpp
 *
 * Purpose:		Test program for Rectangular arrays
 *
 * Created		16th June 2003
 * Updated:		20th June 2003
 *
 * Copyright:	Synesis Software Pty Ltd, (c) 2003. All rights reserved.
 *
 * Home:		www.synesis.com.au/software
 *
 * ////////////////////////////////////////////////////////////////////////// */

#define	NOMINMAX

#include <stdio.h>

#include <stlsoft_fixed_array.h>
#include <stlsoft_simple_string.h>

typedef	stlsoft_ns_qual(basic_simple_string)<char>		string_t;
typedef stlsoft_ns_qual(fixed_array_3d)<string_t>		array_3d_t;


#if defined(WIN32) || \
    defined(_WIN32)

#include <winstl_performance_counter.h>
typedef winstl_ns_qual(performance_counter)		performance_counter_t;

#else

#include <unixstl_performance_counter.h>
typedef unixstl_ns_qual(performance_counter)	performance_counter_t;

#endif /* platform */

/* /////////////////////////////////////////////////////////////////////////////
 * printf() format string
 */

#if defined(__BORLANDC__) || \
    defined(__GNUC__) || \
    defined(_INTEL_COMPILER) || \
    defined(_MSC_VER)
 #define _sint64_fmt    "%I64d"
 #define _uint64_fmt    "%I64u"
#elif defined(__DMC__) || \
      defined(__MWERKS__)
 #define _sint64_fmt    "%lld"
 #define _uint64_fmt    "%llu"
#else
 #error Further compiler discrimination is required
#endif /* compiler */

/* ////////////////////////////////////////////////////////////////////////// */

const int	WARMUPS			=	2;
const int	DEF_ITERATIONS	=	1;
const int	DEF_D0			=	10;
const int	DEF_D1			=	50;
const int	DEF_D2			=	100;

/* ////////////////////////////////////////////////////////////////////////// */

/* Avoid the painful mess that is compiling with max & Windows headers */
inline int int_max(int i1, int i2)
{
	return i1 < i2 ? i2 : i1;
}

/* ////////////////////////////////////////////////////////////////////////// */

int main(int argc, char **argv)
{
	performance_counter_t	counter;
	int						ITERATIONS	=	(argc > 1) ? atoi(argv[1]) : 0;
	int						D0			=	(argc > 2) ? atoi(argv[2]) : 0;
	int						D1			=	(argc > 3) ? atoi(argv[3]) : 0;
	int						D2			=	(argc > 4) ? atoi(argv[4]) : 0;
	int						w;
	int						total		=	0;

	ITERATIONS	=	int_max(ITERATIONS, DEF_ITERATIONS);
	D0			=	int_max(D0, DEF_D0);
	D1			=	int_max(D1, DEF_D1);
	D2			=	int_max(D2, DEF_D2);

	for(w = 0; w < WARMUPS; ++w)
	{
		char	source[9] = "sz000000";
		int		i;

		counter.start();

		{
			array_3d_t	array(D0, D1, D2);

			for(i = 0; i < ITERATIONS; ++i)
			{
				// (i) Create the array
				int	i0;
				int	i1;
				int	i2;

				for(i0 = 0; i0 < D0; ++i0)
				{
					source[3] = 'a' + (i0 & 0x0f);
					source[2] = 'a' + ((i0 & 0xf0) << 16);

					for(i1 = 0; i1 < D1; ++i1)
					{
						source[5] = 'a' + (i1 & 0x0f);
						source[4] = 'a' + ((i1 & 0xf0) << 16);

						for(i2 = 0; i2 < D2; ++i2)
						{
							source[7] = 'a' + (i2 & 0x0f);
							source[6] = 'a' + ((i2 & 0xf0) << 16);

							// Use unchecked efficient access here ...
							array.at_unchecked(i0, i1, i2) = source;

							// ... and checked & indexed access here.
							total += array[i0][i1][i2].length();
						}
					}
				}
			}
		}

		counter.stop();
	}

	fprintf(stdout,
			"rectarr (%s):\t" _sint64_fmt " in %d repeats;\t[%d, %d, %d]: %d\n",
			__STLSOFT_COMPILER_LABEL_STRING,
			counter.get_milliseconds(),
			ITERATIONS,
			D0,
			D1,
			D2,
			total);

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
