/* /////////////////////////////////////////////////////////////////////////////
 * File:		zlibtest.cpp
 *
 * Purpose:		
 *
 * Created		17th June 2003
 * Updated:		20th June 2003
 *
 * Copyright:	Synesis Software Pty Ltd, (c) 2003. All rights reserved.
 *
 * Home:		www.synesis.com.au/software
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include <stdio.h>
#include <windows.h>

#define	_WINSTL_NO_NAMESPACES

#include <winstl_performance_counter.h>

#include "zlib.h"

/* /////////////////////////////////////////////////////////////////////////////
 * printf() format string
 */

#if defined(__BORLANDC__) || \
    defined(__GNUC__) || \
    defined(_INTEL_COMPILER) || \
    defined(_MSC_VER) || \
    defined(__WATCOMC__)
 #define _sint64_fmt    "%I64d"
 #define _uint64_fmt    "%I64u"
#elif defined(__DMC__) || \
      defined(__MWERKS__)
 #define _sint64_fmt    "%lld"
 #define _uint64_fmt    "%llu"
#else
 #error Further compiler discrimination is required
#endif /* compiler */

/* ////////////////////////////////////////////////////////////////////////// */

const int	WARMUPS			=	2;

/* ////////////////////////////////////////////////////////////////////////// */

BOOL SetFileSize(HANDLE hfile, DWORD dwSize)
{
	return SetFilePointer(hfile, dwSize, NULL, FILE_BEGIN) && SetEndOfFile(hfile);
}

int main(int argc, char **argv)
{
	if(argc < 2)
	{
		fprintf(stderr, "%s: USAGE: zlibtest <src-file> [<dest-file>]\n", argv[0]);
	}
	else
	{
		DWORD		dest_flags	=	0;
		char const	*zlver		=	zlibVersion();
		char const	*srcFile	=	argv[1];
		char const	*destFile	=	(argc > 2) ? argv[2] : (dest_flags |= FILE_FLAG_DELETE_ON_CLOSE, tmpnam(NULL));
		HANDLE		hfileSrc;
		HANDLE		hfileDest;

#ifdef __SYNSOFT_DBS_DEBUG
		fprintf(stdout, "zlibtest: %s %s\n", zlver, srcFile);
#endif /* __SYNSOFT_DBS_DEBUG */

		if(INVALID_HANDLE_VALUE == (hfileSrc =	CreateFile	(	srcFile
															,	GENERIC_READ
															,	FILE_SHARE_READ
															,	NULL
															,	OPEN_ALWAYS
															,	FILE_FLAG_RANDOM_ACCESS
															,	NULL)))
		{
			fprintf(stderr, "Could not open file \"%s\" (%d)\n", srcFile, GetLastError());
		}
		else if(INVALID_HANDLE_VALUE == (hfileDest =	CreateFile	(	destFile
																	,	GENERIC_READ | GENERIC_WRITE
																	,	0
																	,	NULL
																	,	CREATE_ALWAYS
																	,	FILE_FLAG_RANDOM_ACCESS | dest_flags
																	,	NULL)))
		{
			fprintf(stderr, "Could not create file \"%s\" (%d)\n", destFile, GetLastError());
		}
		else
		{
			DWORD	srcSize		=	GetFileSize(hfileSrc, NULL);
			DWORD	destSize	=	12 + (2 * srcSize); /* Add the 12 bytes plus 5% (not 0.1) */

			/* Now extend the file */
			if(!SetFileSize(hfileDest, destSize))
			{
				fprintf(stderr, "Could not set size of destination file \"%s\" (%d)\n", destFile, GetLastError());
			}
			else
			{
				/* Now map them */
				HANDLE	hmapSrc;
				HANDLE	hmapDest;

				if(NULL == (hmapSrc = CreateFileMapping(hfileSrc, NULL, PAGE_READONLY, 0, 0, NULL)))
				{
					fprintf(stderr, "Could not create mapping for file \"%s\" (%d)\n", srcFile, GetLastError());
				}
				else if(NULL == (hmapDest = CreateFileMapping(hfileDest, NULL, PAGE_READWRITE, 0, 0, NULL)))
				{
					fprintf(stderr, "Could not create mapping for file \"%s\" (%d)\n", destFile, GetLastError());
				}
				else
				{
					LPCVOID	pcvSrc;
					LPVOID	pvDest;

					if(NULL == (pcvSrc = MapViewOfFile(hmapSrc, FILE_MAP_READ, 0, 0, 0)))
					{
						fprintf(stderr, "Could not map view of file \"%s\" (%d)\n", srcFile, GetLastError());
					}
					else if(NULL == (pvDest = MapViewOfFile(hmapDest, FILE_MAP_WRITE, 0, 0, 0)))
					{
						fprintf(stderr, "Could not map view of file \"%s\" (%d)\n", destFile, GetLastError());
					}
					else
					{
#if 0
						int		i;
						LPBYTE	pyView	=	(LPBYTE)pcvSrc;
						union
						{
							DWORD	dw;
							BYTE	bytes[4];
						}		u;


						for(i = 0, u.dw = 0; i < srcSize; ++i)
						{
//							fprintf(stdout, "%05d %02x\n", i, pyView[i]);

							u.bytes[i & 0x3] ^= pyView[i];
						}

						fprintf(stdout, "%05d 0x%08x\n", i, u.dw);
						fprintf(stderr, "%05d 0x%08x\n", i, u.dw);
#else
						performance_counter	counter;
						uLongf				destLen	=	destSize;
						int					iRes;
						int					w;

						for(w = 0; w < WARMUPS; ++w)
						{
							counter.start();
							iRes = compress2(	(Bytef*)pvDest
											,	&destLen
											,	(Bytef const *)pcvSrc
											,	srcSize
											,	Z_BEST_COMPRESSION);
							counter.stop();
						}

						if(Z_OK != iRes)
						{
							fprintf(stderr, "Compression failed (%d)\n", iRes);
						}
						else
						{
							destSize = destLen;
#ifdef __SYNSOFT_DBS_DEBUG
							fprintf(stdout,
									(argc > 2)
										? "Compressed %d bytes into %d in %ldms. Saved %ld. Written to \"%s\"\n"
										: "Compressed %d bytes into %d in %ldms. Saved %ld\n",
									srcSize,
									destSize,
									(long)counter.get_milliseconds(),
									((long)srcSize - (long)destSize),
									destFile);
#endif /* __SYNSOFT_DBS_DEBUG */

							fprintf(stdout,
									(argc > 2)
										? "zlibtest (%s):\t" _sint64_fmt "\tCompressed %d bytes into %d; saved %ld. Written to \"%s\"\n"
										: "zlibtest (%s):\t" _sint64_fmt "\tCompressed %d bytes into %d; saved %ld\n",
									__STLSOFT_COMPILER_LABEL_STRING,
									counter.get_milliseconds(),
									srcSize,
									destSize,
									((long)srcSize - (long)destSize),
									destFile);
						}
#endif /* 0 */

						UnmapViewOfFile(pvDest);
						UnmapViewOfFile((LPVOID)pcvSrc);
					}

					CloseHandle(hmapDest);
					CloseHandle(hmapSrc);
				}

				SetFileSize(hfileDest, destSize);
				CloseHandle(hfileDest);
				CloseHandle(hfileSrc);
			}
		}
	}

	return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
